@@ -1,10 +1,10 @@
 # @package      hubzero-submit-common
 # @file         CommandParser.py
 # @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012 HUBzero Foundation, LLC.
+# @copyright    Copyright (c) 2012-2013 HUBzero Foundation, LLC.
 # @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
 #
-# Copyright (c) 2012 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2013 HUBzero Foundation, LLC.
 #
 # This file is part of: The HUBzero(R) Platform for Scientific Collaboration
 #
@@ -22,9 +22,11 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+# HUBzero is a registered trademark of HUBzero Foundation, LLC.
+#
+
 from optparse import OptionParser,SUPPRESS_HELP,IndentedHelpFormatter
 import textwrap
-#import sys
 import os.path
 import re
 import math
@@ -36,7 +38,6 @@ from hubzero.submit.LogMessage    import logID as log
 from hubzero.submit.CommentedFile import CommentedFile
 
 class CommandParser:
-
    class IndentedHelpFormatterWithNL(IndentedHelpFormatter):
       def _format_text(self, text):
          """
@@ -109,7 +110,7 @@ class CommandParser:
          return "".join(result)
 
 
-   def __grabMultipleArguments(self,option,opt_str,value,parser):
+   def __grabMultipleStringArguments(self,option,opt_str,value,parser):
       assert value is None
       value = getattr(parser.values,option.dest)
       if not value:
@@ -160,6 +161,63 @@ class CommandParser:
       if len(value) > 0:
          setattr(parser.values,option.dest,value)
 
+   def __grabMultipleJobIdArguments(self,option,opt_str,value,parser):
+      assert value is None
+      value = getattr(parser.values,option.dest)
+      if not value:
+         value = []
+      rargs = parser.rargs
+      while rargs:
+         arg = rargs[0]
+
+         # Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
+         # etc.  Note that this also stops on "-3" or "-3.0", so if
+         # your option takes numeric values, you will need to handle
+         # this.
+         if  (arg == "--"):
+            del rargs[0]
+            break
+         elif((arg[:2] == "--" and len(arg) > 2) or
+              (arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
+            break
+         else:
+            try:
+               jobId = int(arg)
+               if not jobId in value:
+                  value.append(jobId)
+            except:
+               pass
+            del rargs[0]
+
+      setattr(parser.values,option.dest,value)
+
+   def __grabJobIdArgument(self,option,opt_str,value,parser):
+      assert value is None
+      value = getattr(parser.values,option.dest)
+      if not value:
+         value = []
+      rargs = parser.rargs
+      if rargs:
+         arg = rargs[0]
+
+         # Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
+         # etc.  Note that this also stops on "-3" or "-3.0", so if
+         # your option takes numeric values, you will need to handle
+         # this.
+         if((arg[:2] == "--" and len(arg) > 2) or
+            (arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
+            pass
+         else:
+            try:
+               jobId = int(arg)
+               if not jobId in value:
+                  value.append(jobId)
+            except:
+               pass
+            del rargs[0]
+
+      setattr(parser.values,option.dest,value)
+
    def __grabMultipleParameterArguments(self,option,opt_str,value,parser):
       assert value is None
       value = getattr(parser.values,option.dest)
@@ -277,6 +335,70 @@ class CommandParser:
          setattr(parser.values,'helpTools',True)
 
 
+   def __grabMultipleVersionArguments(self,option,opt_str,value,parser,allowedVersionOptions):
+      assert value is None
+      value = []
+      rargs = parser.rargs
+      while rargs:
+         arg = rargs[0]
+
+         # Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
+         # etc.  Note that this also stops on "-3" or "-3.0", so if
+         # your option takes numeric values, you will need to handle
+         # this.
+         if  (arg == "--"):
+            del rargs[0]
+            break
+         elif((arg[:2] == "--" and len(arg) > 2) or
+            (arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
+            break
+         elif arg in allowedVersionOptions:
+            if not arg in value:
+               value.append(arg)
+               del rargs[0]
+         else:
+            break
+
+      if len(value) == 0:
+         setattr(parser.values,'version',True)
+      else:
+         if 'client' in value:
+            setattr(parser.values,'versionClient',True)
+         if 'server' in value:
+            setattr(parser.values,'versionServer',True)
+         if 'distributor' in value:
+            setattr(parser.values,'versionDistributor',True)
+
+
+   def __grabVersionArgument(self,option,opt_str,value,parser,allowedVersionOptions):
+      assert value is None
+      value = []
+      rargs = parser.rargs
+      if rargs:
+         arg = rargs[0]
+
+         # Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
+         # etc.  Note that this also stops on "-3" or "-3.0", so if
+         # your option takes numeric values, you will need to handle
+         # this.
+         if((arg[:2] == "--" and len(arg) > 2) or
+            (arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
+            pass
+         else:
+            if arg in allowedVersionOptions:
+               value.append(arg)
+               del rargs[0]
+
+      if len(value) == 0:
+         setattr(parser.values,'version',True)
+      elif 'client' in value:
+         setattr(parser.values,'versionClient',True)
+      elif 'server' in value:
+         setattr(parser.values,'versionServer',True)
+      elif 'distributor' in value:
+         setattr(parser.values,'versionDistributor',True)
+
+
    def __init__(self,
                 doubleDashTerminator=False):
       self.commandOptions       = None
@@ -350,19 +472,24 @@ submit -s / -p @@address=23 Main St.,Hometown,Indiana/42 Broadway,Hometown,India
 change back to comma for the @@color parameter and any remaining arguments.
 We shouldn't have to change the separator often, but it might come in handy if the value strings themselves have commas.
 \n\n
-submit @@num=1:1000 sim.exe input@@num
+submit -p @@num=1:1000 sim.exe input@@num
 \n\n
 \tSubmit jobs 1,2,3,...,1000.
 Parameter names such as @@num are recognized not only in template files, but also for arguments on the command line.
 In this case, the numbers 1,2,3,...,1000 are substituted into the file name,
 so the various jobs take their input from "input1", "input2", ..., "input1000".
 \n\n
-submit @@file=glob:indeck* sim.exe @:file
+submit -p @@file=glob:indeck* sim.exe @@file
 \n\n
 \tLook for files matching indeck* and use the list of names as the parameter @@file.
 Those values could be substituted into other template files, or used on the command line as in this example.
-Suppose the directory contains files indeckA, indeckB, and indeck-123.
-This example would launch three jobs using each of those files as input for the job.
+Suppose the directory contains files indeck1, indeck10, and indeck2.  The glob option will order the
+files in a natural order: indeck1, indeck2, indeck10.  This example would launch three jobs using each
+of those files as input for the job.
+\n\n
+submit -p @@file=globnat:indeck* sim.exe @@file
+\n\n
+\tThis option has been deprecated.  The functionality is now available with the glob option.
 \n\n
 """
       indentedHelpFormatterWithNL = self.IndentedHelpFormatterWithNL()
@@ -394,16 +521,30 @@ This example would launch three jobs using each of those files as input for the
       self.parser.set_defaults(localExecution=False)
       self.parser.add_option("-l","--local",action="store_true",dest="localExecution", \
                                             help="Execute command locally")
+      if doubleDashTerminator:
+         self.parser.add_option("--status",action="callback", \
+                                           callback=self.__grabMultipleJobIdArguments,dest="statusIds", \
+                                           help="Report status for runs executing remotely.")
+         self.parser.add_option("-k","--kill",action="callback", \
+                                              callback=self.__grabMultipleJobIdArguments,dest="killIds", \
+                                              help="Kill runs executing remotely.")
+      else:
+         self.parser.add_option("--status",action="callback", \
+                                           callback=self.__grabJobIdArgument,dest="statusIds", \
+                                           help="Report status for runs executing remotely.")
+         self.parser.add_option("-k","--kill",action="callback", \
+                                              callback=self.__grabJobIdArgument,dest="killIds", \
+                                              help="Kill runs executing remotely.")
 
       if doubleDashTerminator:
          self.parser.add_option("-v","--venue",action="callback", \
-                                               callback=self.__grabMultipleArguments,dest="destinations", \
+                                               callback=self.__grabMultipleStringArguments,dest="destinations", \
                                                help="Remote job destination list terminated by --")
          self.parser.add_option("-i","--inputfile",action="callback", \
-                                                   callback=self.__grabMultipleArguments,dest="inputFiles", \
+                                                   callback=self.__grabMultipleStringArguments,dest="inputFiles", \
                                                    help="Input file list terminated by --")
          self.parser.add_option("-o","--outputfile",action="callback", \
-                                                    callback=self.__grabMultipleArguments,dest="outputFiles", \
+                                                    callback=self.__grabMultipleStringArguments,dest="outputFiles", \
                                                     help=SUPPRESS_HELP)
          self.parser.add_option("-p","--parameters",action="callback", \
                                                     callback=self.__grabMultipleParameterArguments,dest="parameters", \
@@ -438,14 +579,19 @@ This example would launch three jobs using each of those files as input for the
       self.parser.add_option("-e","--env",action="callback", \
                                           callback=self.__grabStringArgument,dest="environment", \
                                           help="Variable=value")
+      self.parser.add_option("--runName",action="store",type="string",dest="runName", \
+                                         help="Name used for directories and files created during the run." + \
+                                              " Restricted to alphanumeric characters")
       self.parser.add_option("-m","--manager",action="callback", \
                                               callback=self.__grabStringArgument,dest="manager", \
                                               help="Multiprocessor job manager")
       self.parser.add_option("-r","--redundancy",action="store",type="int",dest="nRedundant", \
-                                            help="Number of indentical simulations to execute in parallel")
+                                                 help="Number of indentical simulations to execute in parallel")
       self.parser.set_defaults(metrics=False)
       self.parser.add_option("-M","--metrics",action="store_true",dest="metrics", \
                                               help="Report resource usage on exit")
+      self.parser.set_defaults(detach=False)
+      self.parser.set_defaults(attachId=0L)
       self.parser.set_defaults(waitSwitch=False)
       self.parser.add_option("-W","--wait",action="store_true",dest="waitSwitch", \
                                            help="Wait for reduced job load before submission")
@@ -458,6 +604,39 @@ This example would launch three jobs using each of those files as input for the
       self.parser.add_option("-q","--noquota",action="store_false",dest="quotaCheck", \
                                               help="Do not enforce local user quota on remote execution host")
 
+      self.parser.set_defaults(doHeartbeat=True)
+      self.parser.add_option("--noHeartbeat",action="store_false",dest="doHeartbeat", \
+                                             help=SUPPRESS_HELP)
+      self.parser.set_defaults(disableStateCheck=False)
+      self.parser.add_option("--disableStateCheck",action="store_true",dest="disableStateCheck", \
+                                                   help=SUPPRESS_HELP)
+      self.parser.set_defaults(disableProbeCheck=True)
+#     self.parser.add_option("--disableProbeCheck",action="store_true",dest="disableProbeCheck", \
+#                                                  help=SUPPRESS_HELP)
+      self.parser.add_option("--userLogPath",action="store",type="string",dest="userLogPath", \
+                                             help=SUPPRESS_HELP)
+      self.parser.add_option("--userJobidPath",action="store",type="string",dest="userJobidPath", \
+                                               help=SUPPRESS_HELP)
+      self.parser.add_option("--x509SubmitProxy",action="store",type="string",dest="x509SubmitProxy", \
+                                                 help=SUPPRESS_HELP)
+
+      self.parser.set_defaults(version=False)
+      self.parser.set_defaults(versionClient=False)
+      self.parser.set_defaults(versionServer=False)
+      self.parser.set_defaults(versionDistributor=False)
+      allowedVersionOptions = ['client','server','distributor']
+
+      if doubleDashTerminator:
+         self.parser.add_option("--version",action="callback", \
+                                            callback=self.__grabMultipleVersionArguments,dest="version", \
+                                            callback_kwargs={'allowedVersionOptions':allowedVersionOptions}, \
+                                            help=SUPPRESS_HELP)
+      else:
+         self.parser.add_option("--version",action="callback", \
+                                            callback=self.__grabVersionArgument,dest="version", \
+                                            callback_kwargs={'allowedVersionOptions':allowedVersionOptions}, \
+                                            help=SUPPRESS_HELP)
+
 
    def parseArguments(self,
                       args):
@@ -466,7 +645,7 @@ This example would launch three jobs using each of those files as input for the
       self.commandOptions   = options
       self.commandArguments = []
       for argument in arguments:
-         self.commandArguments.append(argument.replace(' ','\ '))
+         self.commandArguments.append(argument)
 
 
    def getOption(self,
@@ -522,14 +701,138 @@ This example would launch three jobs using each of those files as input for the
       self.parser.print_help()
 
 
+   OPERATIONMODEHELPUSAGE          = 1 << 0
+   OPERATIONMODEHELPTOOLS          = 1 << 1
+   OPERATIONMODEHELPVENUES         = 1 << 2
+   OPERATIONMODEHELPMANAGERS       = 1 << 3
+   OPERATIONMODEVERSIONCLIENT      = 1 << 8
+   OPERATIONMODEVERSIONSERVER      = 1 << 9
+   OPERATIONMODEVERSIONDISTRIBUTOR = 1 << 10
+   OPERATIONMODERUNNONE            = 1 << 15
+   OPERATIONMODERUNSERVER          = 1 << 16
+   OPERATIONMODERUNPROXY           = 1 << 17
+   OPERATIONMODERUNSTATUS          = 1 << 18
+   OPERATIONMODERUNKILL            = 1 << 19
+   OPERATIONMODERUNDISTRIBUTOR     = 1 << 20
+   OPERATIONMODERUNDISTRIBUTORID   = 1 << 21
+
+
+   def getOperationMode(self):
+      operationMode = 0
+      if self.getOption('help'):
+         operationMode = operationMode | self.OPERATIONMODEHELPUSAGE
+      if self.getOption('helpTools'):
+         operationMode = operationMode | self.OPERATIONMODEHELPTOOLS
+      if self.getOption('helpVenues'):
+         operationMode = operationMode | self.OPERATIONMODEHELPVENUES
+      if self.getOption('helpManagers'):
+         operationMode = operationMode | self.OPERATIONMODEHELPMANAGERS
+
+      if self.getOption('version') or self.getOption('versionClient'):
+         operationMode = operationMode | self.OPERATIONMODEVERSIONCLIENT
+      if self.getOption('version') or self.getOption('versionServer'):
+         operationMode = operationMode | self.OPERATIONMODEVERSIONSERVER
+      if self.getOption('version') or self.getOption('versionDistributor'):
+         operationMode = operationMode | self.OPERATIONMODEVERSIONDISTRIBUTOR
+
+      enteredCommand = self.getEnteredCommand()
+      if enteredCommand == "":
+         if   self.getOption('statusIds') != None:
+            operationMode = operationMode | self.OPERATIONMODERUNSTATUS
+         elif self.getOption('killIds') != None:
+            operationMode = operationMode | self.OPERATIONMODERUNKILL
+         elif self.getOption('attachId') != 0L:
+            operationMode = operationMode | self.OPERATIONMODERUNPROXY
+         else:
+            if   operationMode & (self.OPERATIONMODEHELPTOOLS | self.OPERATIONMODEHELPVENUES | \
+                                  self.OPERATIONMODEHELPMANAGERS | self.OPERATIONMODEVERSIONDISTRIBUTOR):
+               operationMode = operationMode | self.OPERATIONMODERUNDISTRIBUTOR
+            elif operationMode & self.OPERATIONMODEVERSIONSERVER:
+               operationMode = operationMode | self.OPERATIONMODERUNSERVER
+            else:
+               operationMode = operationMode | self.OPERATIONMODERUNNONE
+      else:
+         operationMode = operationMode | self.OPERATIONMODERUNDISTRIBUTORID
+
+      return(operationMode)
+
+
    def validateArguments(self):
       validArguments = True
       if self.getOption('localExecution') and self.getOption('destinations'):
-         errorMessage = "Local execution can not be combined with remote destinations"
+         errorMessage = "Local execution cannot be combined with remote destinations"
          log(errorMessage)
-#        sys.stderr.write(errorMessage + "\n")
-#        sys.stderr.flush()
          validArguments = False
+      if self.getOption('localExecution') and self.getOption('detach'):
+         errorMessage = "Local execution cannot run detached"
+         log(errorMessage)
+         validArguments = False
+      if self.getOption('statusIds') != None and self.getOption('detach'):
+         errorMessage = "Status reporting cannot run detached"
+         log(errorMessage)
+         validArguments = False
+      if self.getOption('killIds') != None and self.getOption('detach'):
+         errorMessage = "Run termination cannot run detached"
+         log(errorMessage)
+         validArguments = False
+      if self.getOption('statusIds') != None and self.getOption('attachId'):
+         errorMessage = "Status reporting cannot be combined with attach"
+         log(errorMessage)
+         validArguments = False
+      if self.getOption('killIds') != None and self.getOption('attachId'):
+         errorMessage = "Run termination cannot be combined with attach"
+         log(errorMessage)
+         validArguments = False
+      if self.getOption('help') or self.getOption('helpTools') or \
+         self.getOption('helpVenues') or self.getOption('helpManagers'):
+         if self.getOption('detach'):
+            errorMessage = "Help cannot be combined with the detach option"
+            log(errorMessage)
+            validArguments = False
+         if self.getOption('attachId'):
+            errorMessage = "Help cannot be combined with the attach option"
+            log(errorMessage)
+            validArguments = False
+      if self.getOption('version') or self.getOption('versionClient') or \
+         self.getOption('versionServer') or self.getOption('versionDistributor'):
+         if self.getOption('detach'):
+            errorMessage = "version cannot be combined with the detach option"
+            log(errorMessage)
+            validArguments = False
+         if self.getOption('attachId'):
+            errorMessage = "version cannot be combined with the attach option"
+            log(errorMessage)
+            validArguments = False
+
+      if self.getOption('runName'):
+         runName = self.getOption('runName')
+         if not runName.isalnum():
+            errorMessage = "runName contains non-alphanumeric characters"
+            log(errorMessage)
+            validArguments = False
+
+      if   self.getOption('statusIds') != None and self.getOption('killIds') != None:
+         errorMessage = "Status reporting cannot be combined with run termination"
+         log(errorMessage)
+         validArguments = False
+      elif len(self.commandArguments) > 0:
+         if self.getOption('statusIds') != None:
+            errorMessage = "Status reporting cannot be combined with run submission"
+            log(errorMessage)
+            validArguments = False
+         if self.getOption('killIds') != None:
+            errorMessage = "Run termination cannot be combined with run submission"
+            log(errorMessage)
+            validArguments = False
+         if self.getOption('help') or self.getOption('helpTools') or \
+            self.getOption('helpVenues') or self.getOption('helpManagers'):
+            errorMessage = "Help cannot be combined with run submission"
+            log(errorMessage)
+            validArguments = False
+         if self.getOption('attachId') != 0L:
+            errorMessage = "Run attachment cannot be combined with run submission"
+            log(errorMessage)
+            validArguments = False
       
       return(validArguments)
 
@@ -567,17 +870,37 @@ This example would launch three jobs using each of those files as input for the
       return(parameterVector)
 
 
+   @staticmethod
+   def __natSort(listValues):
+      """ Sort the given list in the way that humans expect.
+      """
+      def tryint(stringValue):
+         try:
+            return int(stringValue)
+         except:
+            return stringValue.lower()
+
+      def alphanumKey(stringValue):
+         """ Turn a string into a list of string and number chunks.
+            "z23a" -> ["z", 23, "a"]
+         """
+         return [ tryint(c) for c in re.split('([0-9]+)',stringValue) ]
+
+      listValues.sort(key=alphanumKey)
+
+
    def parseParameter(self,
                       parameter,
                       separator=""):
       intExpr      = "[0-9]+"
-      numExpr      = "[-+]?(?:[0-9]+\.?[0-9]*|\.[0-9]+)(?:[eEdD]\+?[0-9]+)?"
+      numExpr      = "[-+]?(?:[0-9]+\.?[0-9]*|\.[0-9]+)(?:[eEdD][-+]?[0-9]+)?"
       matlabExpr   = "^(%s)[ \t]*:[ \t]*(%s)[ \t]*(?::[ \t]*(%s))?$" % (numExpr,numExpr,numExpr)
       rangeExpr    = "^(%s)[ \t]*-[ \t]*(%s)$" % (numExpr,numExpr)
       rangeByExpr  = "^(%s)[ \t]*[-:][ \t]*(%s)[ \t]+by[ \t]+(%s)$" % (numExpr,numExpr,numExpr)
       rangePtsExpr = "^(%s)[ \t]*[-:][ \t]*(%s)[ \t]+in[ \t]+(%s)(?:[ \t]*po?i?n?ts)?(?:[ \t]+(line?a?r?|log))?$" % \
                                                                       (numExpr,numExpr,intExpr)
       globExpr     = "^glob:(.+)"
+      globNatExpr  = "^globnat:(.+)"
 
       reIntExpr      = re.compile(intExpr)
       reNumExpr      = re.compile(numExpr)
@@ -586,6 +909,7 @@ This example would launch three jobs using each of those files as input for the
       reRangeByExpr  = re.compile(rangeByExpr)
       reRangePtsExpr = re.compile(rangePtsExpr)
       reGlobExpr     = re.compile(globExpr)
+      reGlobNatExpr  = re.compile(globNatExpr)
 
       if   reMatlabExpr.match(parameter):
       # matched: v1:v2 or v1:v2:v3
@@ -633,7 +957,20 @@ This example would launch three jobs using each of those files as input for the
          basename = os.path.basename(pattern)
          dirFiles = os.listdir(dirname)
          matchingFiles = fnmatch.filter(dirFiles,basename)
-         matchingFiles.sort()
+         self.__natSort(matchingFiles)
+         parameterVector = []
+         for matchingFile in matchingFiles:
+            parameterVector.append(os.path.join(dirname,matchingFile))
+      elif reGlobNatExpr.match(parameter):
+      # look for files matching the globnat pattern
+         pattern = reGlobNatExpr.match(parameter).group(1)
+         dirname = os.path.dirname(pattern)
+         if dirname == "":
+            dirname = os.getcwd()
+         basename = os.path.basename(pattern)
+         dirFiles = os.listdir(dirname)
+         matchingFiles = fnmatch.filter(dirFiles,basename)
+         self.__natSort(matchingFiles)
          parameterVector = []
          for matchingFile in matchingFiles:
             parameterVector.append(os.path.join(dirname,matchingFile))
@@ -664,8 +1001,6 @@ This example would launch three jobs using each of those files as input for the
                   else:
                      errorMessage = "Parameter @@%s defined more than once" % (name)
                      log(errorMessage)
-#                    sys.stderr.write(errorMessage + "\n")
-#                    sys.stderr.flush()
                      exitCode = 1
                elif os.path.isfile(sweepParameter):
                   sep = ','
@@ -686,14 +1021,10 @@ This example would launch three jobs using each of those files as input for the
                                     else:
                                        errorMessage = "Parameter @@%s defined more than once" % (name)
                                        log(errorMessage)
-#                                      sys.stderr.write(errorMessage + "\n")
-#                                      sys.stderr.flush()
                                        exitCode = 1
                                  else:
                                     errorMessage = "Bad parameter definition \"%s\" -- should be @@NAME=expr" % (sweepParameter)
                                     log(errorMessage)
-#                                   sys.stderr.write(errorMessage + "\n")
-#                                   sys.stderr.flush()
                                     exitCode = 1
                            elif separatorPattern.match(record):
                               sep = separatorPattern.match(record).group(1)
@@ -703,8 +1034,6 @@ This example would launch three jobs using each of those files as input for the
                else:
                   errorMessage = "Bad parameter definition \"%s\" -- should be @@NAME=expr or a file" % (sweepParameter)
                   log(errorMessage)
-#                 sys.stderr.write(errorMessage + "\n")
-#                 sys.stderr.flush()
                   exitCode = 1
 
       return(exitCode)
@@ -715,7 +1044,7 @@ This example would launch three jobs using each of those files as input for the
       self.nCSVDataCombinations = 0
       if self.commandOptions.csvData:
          csvDataFile = self.commandOptions.csvData[0]
-         headerExpr   = "^@@([a-zA-Z0-9_]+)"
+         headerExpr   = "^\s*@@([a-zA-Z0-9_]+)\s*$"
          reHeaderExpr = re.compile(headerExpr)
          if os.path.isfile(csvDataFile):
             fpCSVData = open(csvDataFile,'rb')
@@ -741,8 +1070,6 @@ This example would launch three jobs using each of those files as input for the
                   else:
                      errorMessage = "Parameter data file %s: header not properly specified" % (csvDataFile)
                      log(errorMessage)
-#                    sys.stderr.write(errorMessage + "\n")
-#                    sys.stderr.flush()
                      exitCode = 1
 
                   for row in csvReader:
@@ -751,16 +1078,12 @@ This example would launch three jobs using each of those files as input for the
                except csv.Error,e:
                   errorMessage = "Error in parameter data file %s, line %d: %s" % (csvDataFile,csvReader.line_num,e)
                   log(errorMessage)
-#                 sys.stderr.write(errorMessage + "\n")
-#                 sys.stderr.flush()
                   exitCode = 1
 
                fpCSVData.close()
          else:
             errorMessage = "%s is not a file" % (csvDataFile)
             log(errorMessage)
-#           sys.stderr.write(errorMessage + "\n")
-#           sys.stderr.flush()
             exitCode = 1
 
       return(exitCode)
@@ -781,8 +1104,6 @@ This example would launch three jobs using each of those files as input for the
          except csv.Error,e:
             errorMessage = "Error in parameter data file %s, line %d: %s" % (csvDataFile,csvReader.line_num,e)
             log(errorMessage)
-#           sys.stderr.write(errorMessage + "\n")
-#           sys.stderr.flush()
 
 
    def __getCSVDataCombinations(self):
@@ -942,7 +1263,5 @@ This example would launch three jobs using each of those files as input for the
          except csv.Error,e:
             errorMessage = "Error in parameter data file %s, line %d: %s" % (csvPath,csvReader.line_num,e)
             log(errorMessage)
-#           sys.stderr.write(errorMessage + "\n")
-#           sys.stderr.flush()
 
 
