@@ -268,18 +268,24 @@ class MessageCore:
             # Happens on non-blocking TCP socket when there's nothing to read
             break
          except (SSL.WantReadError, SSL.WantWriteError, SSL.WantX509LookupError):
+            # Happens on non-blocking TCP/SSL socket when there's nothing to read
             break
          except SSL.SysCallError, (errnum,errstr):
             if errnum == -1:
+               # Unexpected EOF
                break
          except:
             log("Unexpected error in receiveNonBlockingMessage() " + message)
             log(traceback.format_exc())
+            if not message:
+               message = None
             break
          else:
             if messageChunk:
                message += messageChunk
             else:
+               if not message:
+                  message = None
                break
 
       return(message)
@@ -300,12 +306,14 @@ class MessageCore:
             if messageChunk == "":
                if message != "":
                   log("socket connection broken in receiveBlockingMessage()")
-               message = ""
+               else:
+                  message = None
                break
       except:
          log("Unexpected error in receiveBlockingMessage() " + message)
-         message = ""
          log(traceback.format_exc())
+         if message == "":
+            message = None
 
       return(message)
 
@@ -319,7 +327,6 @@ class MessageCore:
          message = self.__receiveBlockingMessage(channel,messageLength,bufferSize)
       else:
          message = self.__receiveNonBlockingMessage(channel,bufferSize)
-#     log("MessageCore:receiveMessage(%s)" % (message))
 
       return(message)
 
